﻿* Encoding: UTF-8.
*--------------------------------------------------------------
*' SPSS Macro for Performing Meta-analysis
*' Written by David B. Wilson
*' See http://mason.gmu.edu/dwilsonb/spss_macro.html for documentation
*' Version 2021.04.01
*-------------------------------------------------------------- .
set printback=off mprint=off errors=off .
preserve .
define metaanal (
   es=!charend('/')
  /w=!charend('/')
  /ivs= !charend('/')
  /group= !charend('/')
  /model=!default("REML") !charend("/")
  /print = !default('RAW') !charend('/')
  ) .
set mxloop = 100 .
set mdisplay = table .
!if (!model !eq "FE") !then .
  !let !htitle = "'Fixed (common) effect model'" .
!ifend .
!if (!model !eq "DL") !then .
!let !htitle ="'Random effects model, Dersimonian-Laird estimator'" .
!ifend .
!if (!model !eq "HS") !then .
!let !htitle ="'Random effects model,  Hunter-Schmidt estimator'" .
!ifend .
!if (!model !eq "HE") !then .
!let !htitle ="'Random effects model,  Hedges estimator'" .
!ifend .
!if (!model !eq "SJ") !then .
!let !htitle ="'Random effects model,  Sidik-Jonkman estimator'" .
!ifend .
!if (!model !eq "SJIT") !then .
!let !htitle ="'Random effects model,  Sidik-Jonkman iterative estimator'" .
!ifend .
!if (!model !eq "ML") !then .
!let !htitle ="'Random effects model,  Full-information maximum likelihood estimator'" .
!ifend .
!if (!model !eq "REML") !then .
!let !htitle ="'Random effects model,  Restricted maximum likelihood estimator'" .
!ifend .
!if (!model !eq "EB") !then .
!let !htitle ="'Random effects model,  Empirical Bayes estimator'" .
!ifend .

*--------------------------------------------------------------
* If meta-anova, sort data
*-------------------------------------------------------------- .
!if (!group !ne "") !then .
sort cases by !group .
!ifend .

*--------------------------------------------------------------
* Enter matrix mode and get data from active file
*-------------------------------------------------------------- .
matrix .
get data /file * /variables = !es !w !ivs !group /missing omit .

*--------------------------------------------------------------
* Some error checking
*-------------------------------------------------------------- .
get esw /file * /variables = !es !w /missing omit .
compute nvars_esw = 0 .
compute nvars_esw = ncol(esw) .
compute nvars_data = 0 .
compute nvars_data = ncol(data) .

do if cmin(esw(1:nrow(esw),2)) le 0 .
. print /title "*** Output invalid: Inverse variance weights must be > 0." .
end if .

!if (!es !ne "" & !w !ne "") !then .
do if nvars_esw < 2 .
. print /title "*** Check you syntax for a mistyped variable name." .
end if.
!ifend .
!if (!ivs !ne "" | !group !ne "") !then .
do if nvars_data < 3 .
. print /title "*** Check you syntax for a mistyped variable name." .
end if.
!ifend .

!if (!w !eq "") !then .
. print /title "*** You must specify an inverse variance weight (/w = varname)." .
!ifend .
!if (!es !eq "") !then .
. print /title "*** You must specify an effect size." .
!ifend .

do if cmin(esw(1:nrow(esw),2)) > 0 .
do if nvars_esw = 2 .
do if nvars_data > 1 .
!if(!es !ne "" & !w !ne "") !then .

*--------------------------------------------------------------
* create matrix of moderators;
* if no moderators, create intercept
*-------------------------------------------------------------- .
!if (!ivs !eq "" & !group !eq "") !then .
. compute x = make(nrow(data),1,1) .
!else .
. !if (!group !eq "") !then .
. compute x = make(nrow(data),ncol(data)-1,1) .
. compute x(1:nrow(data),2:(ncol(data)-1)) = data(1:nrow(data),3:ncol(data)). 
. !else .
. compute grp = data(1:nrow(data),3).
. compute x = design(grp) .
. compute group = inv(T(x)*x)*T(x)*grp .
.  !ifend .
!ifend .

*--------------------------------------------------------------
* create remaining needed matrices
*-------------------------------------------------------------- .
compute es = data(1:nrow(data),1) .
compute w = data(1:nrow(data),2) .
compute v = (w&**-1) .
compute k = nrow(data) .
compute j = ncol(x) .
release data .

*--------------------------------------------------------------
* start compute tau-squared 
*-------------------------------------------------------------- .
* DL estimator .
!if (!model !eq "DL") !then .
compute wdiag = mdiag(w) .
compute invxwx = inv(t(x)*wdiag*x) .
compute p = wdiag - wdiag * x * invxwx * (t(x)*wdiag) .
compute tracep = trace(p) .
compute rss = t(es) * p * es .
compute tau2 = (rss - (k-j)) / tracep .
!ifend .

* Hunter and Schmidt .
!if (!model !eq "HS") !then .
compute wdiag = mdiag(w) .
compute invxwx = inv(t(x)*wdiag*x) .
compute p = wdiag - wdiag * x * invxwx * (t(x)*wdiag) .
compute tracep = trace(p) .
compute rss = t(es) * p * es .
compute tau2 = (rss - k) / csum(w) .
.do if tau2 < 0 .
. compute tau2 = 0 .
.end if .
!ifend .

* Hedges .
!if (!model !eq "HE" | !model !eq "REML" | !model !eq "ML" | !model !eq "EB") !then .
compute xx = t(x)*x .
compute invxx = inv(t(x)*x) .
compute p = ident(k) - x*(invxx*t(x)) .
compute rss = t(es) * p * es .
compute vdiag = mdiag(v) .
compute pvdiag = p * vdiag .
compute trpv = trace(pvdiag) .
compute tau2 = (rss - trpv) / (k-j) .
!ifend .

* Sidik-Jonkman (SJ) .
!if (!model !eq "SJ" | !model !eq "SJIT") !then .
compute tau2start =  (csum(es&**2) - csum(es)&**2/k)/k .
compute vsj = v+tau2start .
compute wsj = 1/(v+tau2start) .
compute wdiag = mdiag(wsj) .
compute invxwx = inv(t(x)*wdiag*x) .
compute p = wdiag - wdiag * x * invxwx * (t(x)*wdiag) .
compute vdiag = mdiag(v) .
compute pvdiag = p * vdiag .
compute rss = t(es) * p * es .
compute tau2 = (rss*tau2start)/(k-j) .
!ifend .

* Sidik-Jonkman (SJ) (iteractive).
!if (!model !eq "SJIT") !then .
loop i = 1 to 100 .
+ compute oldtau2 = tau2 .
+ compute wsj = 1/(v+oldtau2) .
+ compute wsjdiag = mdiag(wsj) .
+ compute invxwx = inv(t(x)*wsjdiag*x) .
+ compute p = wsjdiag - wsjdiag * x * invxwx * (t(x)*wsjdiag) .
+ compute rss = t(es) * p * es .
+ compute tau2 = (rss*tau2)/(k-j) .
+ compute tauchange = abs(tau2-oldtau2) .
end loop if tauchange < .00001 .
!ifend .

* ML .
!if (!model !eq "ML" | !model !eq "REML" | !model !eq "EB") !then .
loop i = 1 to 100 .
+ compute oldtau2 = tau2 .
+ compute wml = 1/(v+tau2) .
+ compute wmldiag = mdiag(wml) .
+ compute invxwx = inv(t(x)*wmldiag*x) .
+ compute p = wmldiag - wmldiag * x * invxwx * (t(x)*wmldiag) .
+ compute pp = p*p .
!if (!model !eq "ML") !then .
+ compute adj = (t(es) * pp * es - trace(wmldiag))/trace(wmldiag*wmldiag) .
!ifend .
!if (!model !eq "REML") !then .
+ compute adj = (t(es) * pp * es - trace(p))  / trace(pp) .
!ifend .
!if (!model !eq "EB") !then .
+ compute adj = ((t(es) * p * es * k/(k-j)) - k)/trace(wmldiag) .
!ifend .
+ compute tau2 = tau2 + adj .
+ compute tauchange = abs(tau2 - oldtau2) .
end loop if tauchange < .00001 .
!ifend .

!if (!model !eq "FE") !then .
+ compute tau2 = 0 .
!ifend .
.do if tau2 < 0 .
. compute tau2 = 0 .
.end if .

* compute standard error of tau2 .
!if (!model !eq "HS") !then .
. compute setau2  = sqrt(1/csum(w)**2 * (2*(k-j) + 4*tau2*trace(p) + 2*tau2**2*msum(p&*p))) .
!ifend .
!if (!model !eq "HE") !then .
. compute setau2 = sqrt(1/(k-j)**2 * (2*msum(pvdiag&*t(pvdiag)) + 4*tau2*trpv + 2*tau2**2*(k-j))) .
!ifend .
!if (!model !eq "DL") !then .
. compute setau2 = sqrt(1/tracep**2 * (2*(k-j) + 4*tau2*tracep + 2*tau2**2*msum(p&*p))) .
!ifend .
!if (!model !eq "SJ") !then .
. compute setau2 = sqrt(tau2start**2/(k-j)**2 * (2*msum(pvdiag&*t(pvdiag)) + 4*tau2*msum(pvdiag&*p) + 2*tau2**2*msum(p&*p))) .
!ifend .
!if (!model !eq "ML") !then .
. compute wml = 1/(v+tau2) .
. compute setau2 = sqrt(2/csum(wml&**2)) .
!ifend .
!if (!model !eq "REML") !then .
. compute wml = 1/(v+tau2) .
. compute wmldiag = mdiag(wml) .
. compute invxwx = inv(t(x)*wmldiag*x) .
. compute p = wmldiag - wmldiag * x * invxwx * (t(x)*wmldiag) .
. compute setau2 = sqrt(2/msum(p&*p)) .
!ifend .
!if (!model !eq "EB" | !model !eq "SJIT") !then .
. compute wml = 1/(v+tau2) .
. compute setau2 = sqrt(2*k**2/(k-j) / csum(wml)**2) .
!ifend .

*--------------------------------------------------------------
* end compute tau-squared 
*-------------------------------------------------------------- .

*--------------------------------------------------------------
* build results
*-------------------------------------------------------------- .
* create fixed and random effects weights based on selected tau .
!if (!model !eq 'FE') !then .
. compute wre = w .
!else .
. compute wre = 1/(v + tau2) .
!ifend .
compute wfe = 1/v .
compute wrediag = mdiag(wre) .
compute wfediag = mdiag(wfe) .
* some values needed for header info .
compute minw = cmin(wre) .
compute maxw = cmax(wre) .
compute mines = cmin(es) .
compute maxes = cmax(es) .
* compute mean or means, or regression coefficients .
compute bre = inv(t(x)*wrediag*x)*t(x)*wrediag*es .
compute se = sqrt(diag(inv(t(x)*wrediag*x))) .
compute z = bre &/ se .
compute pz =  (1 - cdfnorm(abs(z)))*2 .
compute lb = bre - se*1.959964 .
compute ub = bre + se*1.959964 .
* compute fixed effect b weights (needed below) .
compute bfe = inv(t(x)*wfediag*x)*t(x)*wfediag*es .

*--------------------------------------------------------------
* Q statistics 
*-------------------------------------------------------------- .
compute w1 = make(k,1,1) .
* compute Q-total using fixed effect weights .
compute ywy = (t(es) * wfediag * es) .
compute iwy = (t(w1) * wfediag * es) .
compute iwi = (t(w1) * wfediag * w1) .
compute qt  = ywy - (iwy&**2) * inv(iwi) .
compute dfqt = k - 1 .
compute pqt = 1 - chicdf(qt,dfqt) .
!if (!model !eq "FE" | !model !eq "DL") !then .
. compute i2 = (qt-dfqt)/qt * 100 .
!else .
. compute vtypical = (k-j) /(trace(wfediag - wfediag * x * inv(t(x) * wfediag * x) * (t(x) * wfediag)))  .
. compute i2 = tau2 / (vtypical+tau2) * 100 .
!ifend .

!if (!ivs !ne "" | !group !ne "") !then .
* compute Q-within using fixed effects weight .
compute qe =  (t(es) * wfediag * es) - (2 * t(bfe)  * t(x) * wfediag * es) + (t(bfe) * t(x) * wfediag * x * bfe) .
compute dfqe = k - j .
compute pqe = 1 - chicdf(qe,dfqe) .
* compute Q-between/model using random effects weights .
compute ywy = (t(es) * wrediag * es) .
compute iwy = (t(w1) * wrediag * es) .
compute iwi = (t(w1) * wrediag * w1) .
compute qtre  = ywy - (iwy&**2) * inv(iwi) .
compute qere =  (t(es) * wrediag * es) - (2 * t(bre)  * t(x) * wrediag * es) + (t(bre) * t(x) * wrediag * x * bre) .
compute qm = qtre - qere .
compute dfqm = j - 1 .
compute pqm = 1 - chicdf(qm,dfqm) .
!if (!model !eq "FE" | !model !eq "DL") !then .
. compute i2 = (qe-dfqe)/qe * 100 .
!else .
. compute vtypical = (k-j) /trace(wfediag - wfediag * x * inv(t(x) * wfediag * x) * (t(x) * wfediag))  .
. compute i2 = tau2 / (vtypical+tau2) * 100 .
!ifend .
!ifend .

do if i2<0 .
. compute i2 = 0 .
end if .

!if (!group !ne "") !then .
. compute qegroup = t((t(es&**2) * wfediag * x)  - t(t(x) * wfediag * es)&**2 *  inv(t(x) * wfediag * x)) .
. compute dfqegroup = diag(T(x)*x) -1 .
. compute pqegroup = make(ncol(x),1,-9) .
loop i = 1 to ncol(x) .
do if dfqegroup(i,1) > 0 .
. compute pqegroup(i,1) = 1 - chicdf(qegroup(i,1), dfqegroup(i,1)) .
end if .
end loop .
!ifend .

*--------------------------------------------------------------
*  Create results matrices
*--------------------------------------------------------------.
!if (!model  !eq "FE") !then .
compute header = make(9,1,-9) .
compute header(1,1) = k .
compute header(2,1) = mines .
compute header(3,1) = maxes .
compute header(4,1) = minw .
compute header(5,1) = maxw .
compute header(6,1) = qt .
compute header(7,1) = dfqt .
compute header(8.1) = pqt .
compute header(9.1) = i2 .
print header /format f8.5
   /title !htitle 
   /clabels = "Value"
   /rlabels = "Number of effect sizes (k)" ,
               "Minimum effect size",
               "Maximum effect size",
               "Minimum weight",
               "Maximum weight", 
               "Homogeneity Q (total)",
               "df(Q)",
               "p(Q)",
               "I^2" .
!ifend .

!if (!model  !ne "FE" & !ivs !eq "" & !group !eq "") !then .
compute header = make(12,1,-9) .
compute header(1,1) = k .
compute header(2,1) = mines .
compute header(3,1) = maxes .
compute header(4,1) = minw .
compute header(5,1) = maxw .
compute header(6,1) = qt .
compute header(7,1) = dfqt .
compute header(8.1) = pqt .
compute header(9.1) = i2 .
compute header(10,1) = tau2 .
compute header(11,1) = setau2 .
compute header(12,1) = sqrt(tau2) .
print header /format f8.5
   /title !htitle 
   /clabels = "Value"
   /rlabels = "Number of effect sizes (k)" ,
               "Minimum effect size",
               "Maximum effect size",
               "Minimum weight",
               "Maximum weight", 
               "Homogeneity Q (total)",
               "df(Q)",
               "p(Q)",
               "I^2" ,
               "tau^2" ,
               "se(tau^2)",
               "tau" .
!ifend .

!if (!model  !ne "FE" & (!ivs !ne "" | !group !ne "")) !then .
compute header = make(9,1,-9) .
compute header(1,1) = k .
compute header(2,1) = mines .
compute header(3,1) = maxes .
compute header(4,1) = minw .
compute header(5,1) = maxw .
compute header(6.1) = i2 .
compute header(7,1) = tau2 .
compute header(8,1) = setau2 .
compute header(9,1) = sqrt(tau2) .
print header /format f8.5
   /title !htitle 
   /clabels = "Value"
   /rlabels = "Number of effect sizes (k)" ,
               "Minimum effect size",
               "Maximum effect size",
               "Minimum weight",
               "Maximum weight", 
               "I^2" ,
               "tau^2" ,
               "se(tau^2)" ,
               "tau" .
!ifend .
  
!if (!ivs !eq "" & !group !eq "") !then .
. compute rtable = make(j,6,-9) .
. compute rtable(1:ncol(x),1) = bre .
. compute rtable(1:ncol(x),2) = se .
. compute rtable(1:ncol(x),3) = lb .
. compute rtable(1:ncol(x),4) = ub .
. compute rtable(1:ncol(x),5) = z .
. compute rtable(1:ncol(x),6) = pz .
. print rtable /format f8.4
    /title "Mean Effect Size and Associated Statistics"
    /clabels = "Mean","SE","Lower 95% CI", "Upper 95% CI", "z","p(z)" 
    /rlabels "Total" .
!ifend .
  
!if (!ivs !ne "") !then .
. compute rtable = make(j,6,-9) .
. compute rtable(1:ncol(x),1) = bre .
. compute rtable(1:ncol(x),2) = se .
. compute rtable(1:ncol(x),3) = lb .
. compute rtable(1:ncol(x),4) = ub .
. compute rtable(1:ncol(x),5) = z .
. compute rtable(1:ncol(x),6) = pz .
. print rtable /format f8.4
    /title "Meta-Regression Table"
    /clabels = "Mean","SE","Lower 95% CI", "Upper 95% CI", "z","p(z)" 
    /rlabels "Intercept" !ivs .
!ifend .

!if (!group !ne "") !then .
. compute rtable = make(j,8,-9) .
. compute rtable(1:ncol(x),1) = inv(T(x)*x)*T(x)*grp .
. compute rtable(1:ncol(x),2) = diag(T(x)*x) .
. compute rtable(1:ncol(x),3) = bre .
. compute rtable(1:ncol(x),4) = se .
. compute rtable(1:ncol(x),5) = lb .
. compute rtable(1:ncol(x),6) = ub .
. compute rtable(1:ncol(x),7) = z .
. compute rtable(1:ncol(x),8) = pz .
. print rtable /format f8.4
    /title "Mean Effect Size and Associated Statistics by Group"
    /clabels = "Group", "k", "Mean","SE","Lower 95% CI", "Upper 95% CI", "z","p(z)" 
    /rlabels ' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' '.
!ifend .

 
!if (!ivs !eq "" & !group !eq "" & (!print !eq "EXP" | !print !eq "exp")) !then .
. compute rtablex = make(j,5,-9) .
. compute rtablex(1:ncol(x),1) = exp(bre) .
. compute rtablex(1:ncol(x),2) = exp(lb) .
. compute rtablex(1:ncol(x),3) = exp(ub) .
. compute rtablex(1:ncol(x),4) = z .
. compute rtablex(1:ncol(x),5) = pz .
. print rtablex /format f8.4
    /title "Exponent Transformed Mean Effect Size and Associated Statistics"
    /clabels = "Exp(Mean)","Lower 95% CI", "Upper 95% CI", "z","p(z)" 
    /rlabels "Total" .
!ifend .
 
!if (!ivs !eq "" & !group !eq "" & (!print !eq "IVZR" | !print !eq "ivzr")) !then .
. compute rtablex = make(j,5,-9) .
. compute rtablex(1:ncol(x),1) = (exp(2*bre) - 1)/(exp(2*bre) + 1) .
. compute rtablex(1:ncol(x),2) = exp(2*lb - 1)/exp(2*lb + 1) .
. compute rtablex(1:ncol(x),3) = exp(2*ub - 1)/exp(2*ub + 1) .
. compute rtablex(1:ncol(x),4) = z .
. compute rtablex(1:ncol(x),5) = pz .
. print rtablex /format f8.4
    /title "Inverse Fisher's Zr Transformed Mean Effect Size and Associated Statistics"
    /clabels = "Mean r","Lower 95% CI", "Upper 95% CI", "z","p(z)" 
    /rlabels "Total" .
!ifend .
 
!if (!ivs !eq "" & !group !eq "" & (!print !eq "PROP" | !print !eq "prop")) !then .
. compute rtablex = make(j,5,-9) .
. compute rtablex(1:ncol(x),1) = exp(bre)/(exp(bre) + 1) .
. compute rtablex(1:ncol(x),2) = exp(lb)/(exp(lb) + 1) .
. compute rtablex(1:ncol(x),3) = exp(ub)/(exp(ub) + 1) .
. compute rtablex(1:ncol(x),4) = z .
. compute rtablex(1:ncol(x),5) = pz .
. print rtablex /format f8.4
    /title "Inverse Logit Transformed Mean Effect Size and Associated Statistics"
    /clabels = "Mean Proportion","Lower 95% CI", "Upper 95% CI", "z","p(z)" 
    /rlabels "Total" .
!ifend .

!if (!group !ne "" & (!print !eq "EXP" | !print !eq "exp")) !then .
. compute rtablex = make(j,7,-9) .
. compute rtablex(1:ncol(x),1) = inv(T(x)*x)*T(x)*grp .
. compute rtablex(1:ncol(x),2) = diag(T(x)*x) .
. compute rtablex(1:ncol(x),3) = exp(bre) .
. compute rtablex(1:ncol(x),4) = exp(lb) .
. compute rtablex(1:ncol(x),5) = exp(ub) .
. compute rtablex(1:ncol(x),6) = z .
. compute rtablex(1:ncol(x),7) = pz .
. print rtablex /format f8.4
    /title "Exponent Transformed Mean Effect Size and Associated Statistics by Group"
    /clabels = "Group", "k", "Exp(Mean)","Lower 95% CI", "Upper 95% CI", "z","p(z)" 
    /rlabels ' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' '.
!ifend .

!if (!group !ne "" & (!print !eq "IVZR" | !print !eq "ivzr")) !then .
. compute rtablex = make(j,7,-9) .
. compute rtablex(1:ncol(x),1) = inv(T(x)*x)*T(x)*grp .
. compute rtablex(1:ncol(x),2) = diag(T(x)*x) .
. compute rtablex(1:ncol(x),3) = (exp(2*bre) - 1)/(exp(2*bre) + 1) .
. compute rtablex(1:ncol(x),4) = exp(2*lb - 1)/exp(2*lb + 1) .
. compute rtablex(1:ncol(x),5) = exp(2*ub - 1)/exp(2*ub + 1) .
. compute rtablex(1:ncol(x),6) = z .
. compute rtablex(1:ncol(x),7) = pz .
. print rtablex /format f8.4
    /title "Inverse Fisher's Zr Transformed Mean Effect Size and Associated Statistics by Group"
    /clabels = "Group", "k", "Mean r","Lower 95% CI", "Upper 95% CI", "z","p(z)" 
    /rlabels ' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' '.
!ifend .

!if (!group !ne "" & (!print !eq "PROP" | !print !eq "prop")) !then .
. compute rtablex = make(j,7,-9) .
. compute rtablex(1:ncol(x),1) = inv(T(x)*x)*T(x)*grp .
. compute rtablex(1:ncol(x),2) = diag(T(x)*x) .
. compute rtablex(1:ncol(x),3) = exp(bre)/(exp(bre) + 1) .
. compute rtablex(1:ncol(x),4) = exp(lb)/(exp(lb) + 1) .
. compute rtablex(1:ncol(x),5) = exp(ub)/(exp(ub) + 1) .
. compute rtablex(1:ncol(x),6) = z .
. compute rtablex(1:ncol(x),7) = pz .
. print rtablex /format f8.4
    /title "Inverse Logit Transformed Mean Effect Size and Associated Statistics by Group"
    /clabels = "Group", "k", "Mean Proportion","Lower 95% CI", "Upper 95% CI", "z","p(z)" 
    /rlabels ' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' '.
!ifend .

*--------------------------------------------------------------
*  Model Sig tests (Q-between etc.)
*--------------------------------------------------------------.
!if (!ivs !ne "" | !group !ne "") !then .
. compute qtable = make(2,3,-9) .
. compute qtable(1,1) = qm .
. compute qtable(1,2) = dfqm .
. compute qtable(1,3) = pqm .
. compute qtable(2,1) = qe .
. compute qtable(2,2) = dfqe .
. compute qtable(2,3) = pqe .
print qtable  /format f8.5
  /title "Q-model/between and Q-residual/within"
  /clabels "Q" "df" "p(Q)" 
  /rlabels "Q-model/between" "Q-residual/within" .
!ifend .

!if (!group !ne "") !then .
. compute qgrptable = make(j,4,-9) .
. compute qgrptable(1:j,1) = inv(T(x)*x)*T(x)*grp .
. compute qgrptable(1:j,2) = qegroup .
. compute qgrptable(1:j,3) = dfqegroup .
. compute qgrptable(1:j,4) = pqegroup .
. print qgrptable /format f8.4
    /title "Partition of Q-residual/within by Group (fixed-effect weights)"
    /clabels = "Group", "Q-within", "df","z(Q)"
    /rlabels ' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' '.
!ifend .

!ifend .
end if .
end if .
end if .

end matrix .

!enddefine
restore 


